/*
 * Common functions
 */

/* load node modules*/
var fs = require('fs');
var path = require('path');
var os = require('os');

// The DeskApp global object
var DA = {
	// windows
	winWelcome: undefined,
	winWebApp: undefined,
	winPreferences: undefined,
	// other
	notifier: undefined,
	zoomNotification: undefined,
	// OS
	isLinux: /^linux/.test(process.platform),
	isWin: /^win/.test(process.platform),
	isMacOS: /^darwin/.test(process.platform),
	isWin7OrHigher: function() {
		return this.isWin && this.isOfMinimumVersion('6.1.7600');
	},
	isWin10OrHigher: function() {
		return this.isWin && this.isOfMinimumVersion('10.0.0');
	},

	// Desktop Environments
	isKDE: process.env.KDE_FULL_SESSION,

	/**
	 * Checks if the version of windows is higher or equal than a specified version
	 * @param {String} minimumVersion version to compare against the windows version
	 * @return {String} true if higher of equal than minimumVersion, else false
	 */
	isOfMinimumVersion: function(minimumVersion) {
		/* Thanks to: https://www.packtpub.com/books/content/platform-detection-your-nwjs-app */
		var actualVersionPieces = os.release().split('.'),
			pieces = minimumVersion.split('.'),
			numberOfPieces = pieces.length;

		for (var i = 0; i < numberOfPieces; i++) {
			var piece = parseInt(pieces[i], 10),
				actualPiece = parseInt(actualVersionPieces[i], 10);

			if (typeof actualPiece === 'undefined') {
				break; /* e.g. 13.1 passed and actual is 13.1.0 */
			} else if (actualPiece > piece) {
				break; /* doesn't matter what the next bits are, the major version (or whatever) is larger */
			} else if (actualPiece === piece) {
				continue; /* to check next version piece */
			} else {
				return false;
			}
		}
		return true; /* all was ok */
	},

	/**
	 * Read settings from local storage
	 */
	readSettings: function() {
		/* First check if old config needs to be converted */
		if (localStorage.getItem('url')) {
			this.convertSettings();
		}

		/* Read Settings or create default */
		global.settings = JSON.parse(localStorage.getItem('settings')) || {
			'general': {},
			'profiles': {},
			'language': {},
			'downloads': {
				'downloadBehavior': 'ask'
			},
		};
		/* Migrate the old hideOnMinimize */
		if (localStorage.getItem('hideOnMinimize') !== null) {
			global.settings.general.hideOnMinimize =
				localStorage.getItem('hideOnMinimize') === 'true';
			localStorage.removeItem('hideOnMinimize');
			this.saveSettings();
		}
		/* Old profiles might not have a download setting at all */
		if (!global.settings.downloads) {
			global.settings.downloads = {
				'downloadBehavior': 'ask'
			};
			this.saveSettings();
		}
		/* Migrate the downloads.openOnCompleted setting */
		if (global.settings.downloads.openOnCompleted &&
			typeof global.settings.downloads.openOnCompleted === 'boolean') {
			global.settings.downloads.downloadBehavior = global.settings.downloads.openOnCompleted ?
				'open' : 'ask';
			delete global.settings.downloads.openOnCompleted;
			this.saveSettings();
		}
	},

	/**
	 * Convert old settings to the new format.
	 * Remove them after conversion
	 */
	convertSettings: function() {
		var url = localStorage.getItem('url'),
			username = localStorage.getItem('username'),
			password = localStorage.getItem('password'),
			authType = localStorage.getItem('authType'),
			webappVersion = localStorage.getItem('webappVersion'),
			zoomFactor = localStorage.getItem('zoomFactor'),
			zoomLevel = localStorage.getItem('zoomLevel');

		var converted_profile = {
			'general': {
				'defaultProfile': 'profile_1',
				'autoLogin': true,
				'lastWebappVersion': webappVersion,
				'zoomFactor': zoomFactor || 100,
				'zoomLevel': zoomLevel || 0
			},
			'profiles': {
				'profile_1': {
					'id': 'profile_1',
					'name': username,
					'email': '',
					'passwordEncryption': DAEncryption.getVersion(),
					'requestOptions': {
						'url': url,
						'form': {
							'username': username,
							'password': DAEncryption.encrypt(password)
						}
					},
					'authType': authType
				}
			}
		};

		localStorage.clear();
		localStorage.setItem('settings', JSON.stringify(converted_profile));
	},

	/**
	 * Save the settings to the local storage and update the global reference
	 * @param {Object} settings object holding the settings
	 + @param {Boolean} showNotifier show a save notification
	 */
	saveSettings: function(settings, showNotifier = false) {
		if (!global.settings) {
			return;
		}
		if (showNotifier && this.winPreferences) {
			var notification = this.requireCustomModule('notifications');
			notification.showSaveNotification(this.winPreferences.window.document.body);
		}
		var newSettings = (settings ? this.mergeSettings(global.settings, settings) : global.settings);

		/*
		 * Remove all values that shouldn't be saved.
		 *  - webapp passwords with remember option disabled
		 *  - request proxy settings
		 */
		for (let profileId in newSettings.profiles) {
			if (newSettings.profiles.hasOwnProperty(profileId)) {
				if (!newSettings.profiles[profileId].rememberPassword &&
					newSettings.profiles[profileId].authType !== 'sso') {
					newSettings.profiles[profileId].requestOptions.form.password = '';
				}
				if (newSettings.profiles[profileId].requestOptions.hasOwnProperty('proxy')) {
					delete newSettings.profiles[profileId].requestOptions.proxy;
				}
			}
		}

		localStorage.setItem('settings', JSON.stringify(newSettings));
		global.settings = newSettings;
	},


	/**
	 * Remove all children of a DOM node.
	 * @param {HTMLElement} parent node to remove the children from
	 */
	removeAllChildren: function(parent) {
		while (parent.firstChild) {
			parent.removeChild(parent.firstChild);
		}
	},

	/**
	 * Recursively merge properties of two objects
	 * @param {Object} obj1 first Object
	 * @param {Object} obj2 second Object
	 */
	mergeSettings: function(obj1, obj2) {
		for (var p in obj2) {
			if (obj1[p]) {
				// Property in destination object set; update its value.
				if (typeof obj2[p] === 'object') {
					obj1[p] = this.mergeSettings(obj1[p], obj2[p]);
				} else {
					obj1[p] = obj2[p];
				}
			} else {
				// Property in destination object not set; create it and set its value.
				obj1[p] = obj2[p];
			}
		}
		return obj1;
	},

	/**
	 * Add a form element to parent
	 * @param {HTMLElement} parent HTMLElement the form should be added
	 * @param {Object} config form configuration Object
	 * @param {Function} submitListener function to call when the submitbutton is clicked
	 */
	addForm: function(parent, config, submitListener) {
		this.removeAllChildren(parent);
		var form = document.createElement("form");
		form.setAttribute('id', config['ID']);
		for (var item in config) {
			if (item === 'ID') {
				continue;
			}
			var current = config[item];
			var el = current['element'];
			var el = document.createElement(el);
			var attributes = current['attributes'];
			for (var attribute in attributes) {
				el.setAttribute(attribute, attributes[attribute]);
			}
			if (current.hasOwnProperty('label')) {
				var label = document.createElement("label");
				label.innerHTML = current['label'];
				label.htmlFor = el.id;
				form.appendChild(label);
			}
			if (current.hasOwnProperty('innerHTML')) {
				el.innerHTML = current['innerHTML'];
			}
			if (current.hasOwnProperty('checked')) {
				el.checked = current['checked'];
			}
			if (current.hasOwnProperty('disabled')) {
				el.disabled = current['disabled'];
			}
			if (current.hasOwnProperty('options')) {
				for (var option in current.options) {
					var opt = document.createElement("option");
					opt.text = current.options[option];
					opt.value = option;
					el.options.add(opt);
				}
			}
			form.appendChild(el);
			if (current.hasOwnProperty('listeners')) {
				var listeners = current['listeners'];
				for (var listener in listeners) {
					el.addEventListener(listener, listeners[listener]);
				}
			}
		}
		form.addEventListener('submit', submitListener);
		parent.appendChild(form);
	},

	/**
	 * Set a profile as default
	 * @param {String} profileId id of the profile to be set as default
	 */
	setAsDefault: function(profileId) {
		global.settings.general.defaultProfile = profileId;
		localStorage.setItem('settings', JSON.stringify(global.settings));
	},

	/**
	 * Calculate an id for a new profile id
	 * @return {Int} integer for the new profile
	 */
	newProfileID: function() {
		var max = 0,
			profiles = global.settings.profiles;
		for (var profile in profiles) {
			var cur = parseInt(profiles[profile].id.split('_')[1]);
			if (cur > max) {
				max = cur;
			}
		}
		return max + 1;
	},

	/**
	 * Saves data in a file with the given name in the local App folder
	 * @param {String} fileName The name of the file in which the data
	 * will be stored.
	 * @param {Object} data The data that needs to be stored
	 */
	saveAppData: function(fileName, data) {
		var filePath = path.join(nw.App.dataPath, fileName + '.json');

		try {
			fs.writeFileSync(filePath, JSON.stringify(data));
		} catch (e) {}
	},

	/**
	 * Returns the data from the given file. The data should be stored
	 * in JSON format, and will be returned in parsed form
	 * @param {String} fileName The file from which the data will be read
	 * @return {Mixed} The parsed JSON data from the file
	 */
	readAppData: function(fileName) {
		var filePath = path.join(nw.App.dataPath, fileName + '.json');
		var data;

		try {
			data = fs.readFileSync(filePath);
			return JSON.parse(data);
		} catch (e) {}

		// If we get here, the file was not found or the data was not correct
		// Lets return undefined
		return;
	},

	/**
	 * Calculate the zoom level from zoom factor.
	 * factor =  = 1.2^level
	 * @param {Int} factor zoom factor
	 * @return {Float} zoom level
	 */
	zoomFactorToLevel: function(factor) {
		return Math.log(factor / 100) / Math.log(1.2);
	},

	/**
	 * Require a custom node module by it's name
	 * @param {String} name of the module to load
	 * @return {Object} the node module
	 */
	requireCustomModule: function(name) {
		return require(path.join(process.cwd(), 'app', 'modules', name + '.js'));
	},

	/**
	 * Open the general profile settings section
	 * @param {Event} event event calling the function
	 */
	openSettings: function(event) {
		if (event) {
			event.preventDefault();
		}
		global.DA.openSettingsWindow("general");
	},

	/**
	 * Create and show the settings window and select the given menuItem.
	 * @param {String} menuItem id of the menuItem to show
	 * @param {Boolean} firstProfile if true open the settings window to create one
	 */
	openSettingsWindow: function(menuItem, createProfile = false) {
		var options = {
			'title': 'DeskApp Settings',
			'width': 800,
			'height': 500,
			'min_height': 500,
			'min_width': 800,
			'id': 'prefs',
			'show': true,
			'frame': true,
			'position': 'center',
		};

		nw.Window.open('app/windows/settings/html/settings.html', options, function(win) {
			global.DA.winPreferences = win;
			/* This is executed when the settings have been updated */
			win.onSettingsChange = function() {
				if (Object.keys(global.settings.profiles).length === 0) {
					global.DA.winWelcome.window.showFirstProfileForm();
				} else {
					global.DA.winWelcome.window.showWelcomeForm();
				}
			};
			win.on('loaded', function() {
				if (createProfile) {
					win.window.newProfile();
				} else if (menuItem) {
					win.window.document.getElementById(menuItem).click();
				}
			});
		});
	}
};
