/*
 * DownloadManager
 */

var DownloadManager = {};

/**
 * Get a Download item by its id.
 * @param {Number} id id of the Download item to get
 * @return {DownloadItem} the Download item
 */
DownloadManager.getItem = function(id) {
	var itemDiv = document.getElementById('item' + id);
	return itemDiv ? itemDiv.item : null;
};

/**
 * Get or create a Download item if not yet existing.
 * @return {DownloadItem} the Download item
 */
DownloadManager.getOrCreate = function(data) {
	var item = DownloadManager.getItem(data.id);
	return item ? item : new DownloadItem(data);
};

/**
 * Loop through all Download items and call the given callback for each.
 * @cb {Function} cb callback function to call for each item
 */
DownloadManager.forEachItem = function(cb) {
	// Calls cb(item, index) in the order that they are displayed, i.e. in order
	// of decreasing startTime.
	Array.from(document.getElementsByClassName('download-item')).forEach(
		function(itemDiv, index) { cb(itemDiv.item, index); });
};

/**
 * Starts polling the progress for all Download items that are in progress.
 */
DownloadManager.startPollingProgress = function() {
	if (DownloadManager.startPollingProgress.tid < 0) {
		DownloadManager.startPollingProgress.tid = setTimeout(
			DownloadManager.startPollingProgress.pollProgress,
			DownloadManager.startPollingProgress.MS);
	}
}
DownloadManager.startPollingProgress.MS = 200;
DownloadManager.startPollingProgress.tid = -1;
DownloadManager.startPollingProgress.pollProgress = function() {
	DownloadManager.startPollingProgress.tid = -1;
	chrome.downloads.search({state: 'in_progress', paused: false},
		function(results) {
		if (!results.length) {
			return;
		}
		results.forEach(function(result) {
			var item = DownloadManager.getOrCreate(result);
			for (var prop in result) {
				item[prop] = result[prop];
			}
			item.render();
			if ((item.state == 'in_progress') && !item.paused) {
				DownloadManager.startPollingProgress();
			}
		});
	});
};

/**
 * Show new Download items.
 */
DownloadManager.showNew = function() {
	var anyItems = (document.getElementsByClassName('download-item').length > 0);

	if (!anyItems) {
		DownloadManager.showNoHistory();
		return;
	}

	DownloadManager.showItems();

	DownloadManager.forEachItem(function(item, index) {
		item.div.hidden = false;
		item.itemGroup.hidden = false;
	});

	document.getElementById('search-input').focus();
};

/**
 * Handler for search requests.
 * Hides all items that are not in the search result.
 */
DownloadManager.onSearch = function() {
	// split string by space, but ignore space in quotes
	// http://stackoverflow.com/questions/16261635
	var query = document.getElementById('search-input').value.match(/(?:[^\s"]+|"[^"]*")+/g);
	if (!query) {
		DownloadManager.showNew();
	} else {
		query = query.map(function(term) {
			// strip quotes
			return (term.match(/\s/) &&
							term[0].match(/["']/) &&
							term[term.length - 1] == term[0]) ?
				term.substr(1, term.length - 2) : term;
		});
		DownloadManager.showLoading();
		chrome.downloads.search({query: query}, function(results) {
			DownloadManager.forEachItem(function(item) {
				item.div.hidden = true;
				item.itemGroup.hidden = true;
			});
			results.forEach(function(result) {
				var rItem = DownloadManager.getOrCreate(result);
				rItem.div.hidden = false;
				rItem.itemGroup.hidden = false;
			});
			if (results.length == 0) {
				DownloadManager.showEmptySearch();
			} else {
				DownloadManager.showItems();
			}
		});
	}
};

/**
 * Load all available Download items.
 */
DownloadManager.loadItems = function() {
	chrome.downloads.search({
			orderBy: ['-startTime']},
		function(results) {
			DownloadManager.loadItems.items = results;
			DownloadManager.loadItems.onLoaded();
	});
};
DownloadManager.loadItems.items = [];
DownloadManager.loadItems.windowLoaded = false;


DownloadManager.loadItems.onLoaded = function() {
	if (!DownloadManager.loadItems.windowLoaded) {
		return;
	}
	DownloadManager.loadItems.items.forEach(function(item) {
		DownloadManager.getOrCreate(item);
	});
	DownloadManager.loadItems.items = [];
	DownloadManager.showNew();
	DownloadManager.startPollingProgress();
};

DownloadManager.loadItems.onWindowLoaded = function() {
	DownloadManager.loadItems.windowLoaded = true;
	DownloadManager.loadItems();
};


/**
 * The different views of the Download history.
 */
DownloadManager.divItems = null;
DownloadManager.divEmpty = null;
DownloadManager.divLoading = null;

DownloadManager.showNoHistory = function() {
	DownloadManager.divEmpty.innerHTML = gt.gettext("There are no items in the download history.");
	DownloadManager.divEmpty.style.display = 'inline-block';
	DownloadManager.divLoading.style.display = 'none';
	DownloadManager.divItems.style.display = 'none';
}

DownloadManager.showEmptySearch = function() {
	DownloadManager.divEmpty.innerHTML = gt.gettext("There are no search results.");
	DownloadManager.divEmpty.style.display = 'inline-block';
	DownloadManager.divLoading.style.display = 'none';
	DownloadManager.divItems.style.display = 'none';
}

DownloadManager.showLoading = function() {
	DownloadManager.divLoading.style.display = 'inline-block';
	DownloadManager.divEmpty.style.display = 'none';
	DownloadManager.divItems.style.display = 'none';
}

DownloadManager.showItems = function() {
	DownloadManager.divLoading.style.display = 'none';
	DownloadManager.divEmpty.style.display = 'none';
	DownloadManager.divItems.style.display = 'inline-block';
}

/**
 * Possible errors of a Download item.
 */
DownloadManager.errorMessages = {
	'FILE_FAILED': gt.gettext('Failed'),
	'FILE_ACCESS_DENIED': gt.gettext('Access Denied'),
	'FILE_NO_SPACE': gt.gettext('No space'),
	'FILE_NAME_TOO_LONG': gt.gettext('Filename too long'),
	'FILE_TOO_LARGE': gt.gettext('File too large'),
	'FILE_VIRUS_INFECTED': gt.gettext('Virus infected'),
	'FILE_TRANSIENT_ERROR': gt.gettext('Transient error'),
	'FILE_BLOCKED': gt.gettext('Blocked'),
	'FILE_SECURITY_CHECK_FAILED': gt.gettext('Security check failed'),
	'FILE_TOO_SHORT': gt.gettext('File too short'),
	'FILE_HASH_MISMATCH': gt.gettext('Hash mismatch'),
	'NETWORK_FAILED': gt.gettext('Network failed'),
	'NETWORK_TIMEOUT': gt.gettext('Network timeout'),
	'NETWORK_DISCONNECTED': gt.gettext('Network disconnected'),
	'NETWORK_SERVER_DOWN': gt.gettext('Server down'),
	'NETWORK_INVALID_REQUEST': gt.gettext('Invalid request'),
	'SERVER_FAILED': gt.gettext('Server failed'),
	'SERVER_NO_RANGE': gt.gettext('SERVER_NO_RANGE'),
	'SERVER_BAD_CONTENT': gt.gettext('Bad content'),
	'SERVER_UNAUTHORIZED': gt.gettext('Unauthorized'),
	'SERVER_CERT_PROBLEM': gt.gettext('Certificate problem'),
	'SERVER_FORBIDDEN': gt.gettext('Forbidden'),
	'SERVER_UNREACHABLE': gt.gettext('Server unreachable'),
	'USER_CANCELED': gt.gettext('Canceled'),
	'USER_SHUTDOWN': gt.gettext('Shutdown'),
	'CRASH': gt.gettext('Crash')
}

/**
 * Format bytes to a human readable format.
 * @param (Number) n number of bytes
 * @return {String} formatted presentation of the bytes
 */
DownloadManager.formatBytes = function(n) {
	if (n < 1024) {
		return n + 'B';
	}
	var prefixes = 'KMGTPEZY';
	var mul = 1024;
	for (var i = 0; i < prefixes.length; ++i) {
		if (n < (1024 * mul)) {
			return (parseInt(n / mul) + '.' + parseInt(10 * ((n / mul) % 1)) + ' ' +
							prefixes[i] + 'iB');
		}
		mul *= 1024;
	}
	return '!!!';
}

/**
 * Format the time left until a Download has finished
 * @param {Number} ms millisenconds to format
 * @return {String} nicer presentation of ms
 */
DownloadManager.formatTimeLeft = function(ms) {
	if (ms < 1000) {
		return gt.gettext('Finishing');
	}
	var days = parseInt(ms / (24 * 60 * 60 * 1000));
	if (days) {
		return days + ' ' + gt.gettext('days left');
	}
	var hours = parseInt(ms / (60 * 60 * 1000)) % 24;
	if (hours) {
		return hours + ' ' + gt.gettext('hours left');
	}
	var minutes = parseInt(ms / (60 * 1000)) % 60;
	if (minutes) {
		return minutes + ' ' + gt.gettext('mins left');
	}
	var seconds = parseInt(ms / 1000) % 60;
	return seconds + ' ' + gt.gettext('sec left');
}

/**
 * Clear the whole list
 */
DownloadManager.clearList = function() {
	DownloadManager.forEachItem(function(item, index) {
		item.erase()
	});
}

